# coding: utf-8

"""
    Bandwidth

    Bandwidth's Communication APIs

    The version of the OpenAPI document: 1.0.0
    Contact: letstalk@bandwidth.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Any, Dict, Optional
from pydantic import BaseModel, Field, StrictStr
from bandwidth.models.call_direction_enum import CallDirectionEnum
from bandwidth.models.machine_detection_result import MachineDetectionResult

class AnswerCallback(BaseModel):
    """
    The Answer event is sent to the answerUrl specified in the createCall request when an outbound call is answered.
    """
    event_type: Optional[StrictStr] = Field(None, alias="eventType", description="The event type, value can be one of the following: answer, bridgeComplete, bridgeTargetComplete, conferenceCreated, conferenceRedirect, conferenceMemberJoin, conferenceMemberExit, conferenceCompleted, conferenceRecordingAvailable, disconnect, dtmf, gather, initiate, machineDetectionComplete, recordingComplete, recordingAvailable, redirect, transcriptionAvailable, transferAnswer, transferComplete, transferDisconnect.")
    event_time: Optional[datetime] = Field(None, alias="eventTime", description="The approximate UTC date and time when the event was generated by the Bandwidth server, in ISO 8601 format. This may not be exactly the time of event execution.")
    account_id: Optional[StrictStr] = Field(None, alias="accountId", description="The user account associated with the call.")
    application_id: Optional[StrictStr] = Field(None, alias="applicationId", description="The id of the application associated with the call.")
    var_from: Optional[StrictStr] = Field(None, alias="from", description="The provided identifier of the caller: can be a phone number in E.164 format (e.g. +15555555555) or one of Private, Restricted, Unavailable, or Anonymous.")
    to: Optional[StrictStr] = Field(None, description="The phone number that received the call, in E.164 format (e.g. +15555555555).")
    direction: Optional[CallDirectionEnum] = None
    call_id: Optional[StrictStr] = Field(None, alias="callId", description="The call id associated with the event.")
    call_url: Optional[StrictStr] = Field(None, alias="callUrl", description="The URL of the call associated with the event.")
    enqueued_time: Optional[datetime] = Field(None, alias="enqueuedTime", description="(optional) If call queueing is enabled and this is an outbound call, time the call was queued, in ISO 8601 format.")
    start_time: Optional[datetime] = Field(None, alias="startTime", description="Time the call was started, in ISO 8601 format.")
    answer_time: Optional[datetime] = Field(None, alias="answerTime", description="Time the call was answered, in ISO 8601 format.")
    tag: Optional[StrictStr] = Field(None, description="(optional) The tag specified on call creation. If no tag was specified or it was previously cleared, this field will not be present.")
    machine_detection_result: Optional[MachineDetectionResult] = Field(None, alias="machineDetectionResult")
    additional_properties: Dict[str, Any] = {}
    __properties = ["eventType", "eventTime", "accountId", "applicationId", "from", "to", "direction", "callId", "callUrl", "enqueuedTime", "startTime", "answerTime", "tag", "machineDetectionResult"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> AnswerCallback:
        """Create an instance of AnswerCallback from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of machine_detection_result
        if self.machine_detection_result:
            _dict['machineDetectionResult'] = self.machine_detection_result.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        # set to None if enqueued_time (nullable) is None
        # and __fields_set__ contains the field
        if self.enqueued_time is None and "enqueued_time" in self.__fields_set__:
            _dict['enqueuedTime'] = None

        # set to None if answer_time (nullable) is None
        # and __fields_set__ contains the field
        if self.answer_time is None and "answer_time" in self.__fields_set__:
            _dict['answerTime'] = None

        # set to None if tag (nullable) is None
        # and __fields_set__ contains the field
        if self.tag is None and "tag" in self.__fields_set__:
            _dict['tag'] = None

        # set to None if machine_detection_result (nullable) is None
        # and __fields_set__ contains the field
        if self.machine_detection_result is None and "machine_detection_result" in self.__fields_set__:
            _dict['machineDetectionResult'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> AnswerCallback:
        """Create an instance of AnswerCallback from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return AnswerCallback.parse_obj(obj)

        _obj = AnswerCallback.parse_obj({
            "event_type": obj.get("eventType"),
            "event_time": obj.get("eventTime"),
            "account_id": obj.get("accountId"),
            "application_id": obj.get("applicationId"),
            "var_from": obj.get("from"),
            "to": obj.get("to"),
            "direction": obj.get("direction"),
            "call_id": obj.get("callId"),
            "call_url": obj.get("callUrl"),
            "enqueued_time": obj.get("enqueuedTime"),
            "start_time": obj.get("startTime"),
            "answer_time": obj.get("answerTime"),
            "tag": obj.get("tag"),
            "machine_detection_result": MachineDetectionResult.from_dict(obj.get("machineDetectionResult")) if obj.get("machineDetectionResult") is not None else None
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


