# coding: utf-8

"""
    Bandwidth

    Bandwidth's Communication APIs

    The version of the OpenAPI document: 1.0.0
    Contact: letstalk@bandwidth.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, Optional, Union
from pydantic import BaseModel, Field, StrictBool, StrictFloat, StrictInt, constr
from bandwidth.models.callback_method_enum import CallbackMethodEnum
from bandwidth.models.machine_detection_mode_enum import MachineDetectionModeEnum

class MachineDetectionConfiguration(BaseModel):
    """
    The machine detection request used to perform <a href='/docs/voice/guides/machineDetection'>machine detection</a> on the call.
    """
    mode: Optional[MachineDetectionModeEnum] = None
    detection_timeout: Optional[Union[StrictFloat, StrictInt]] = Field(15, alias="detectionTimeout", description="The timeout used for the whole operation, in seconds. If no result is determined in this period, a callback with a `timeout` result is sent.")
    silence_timeout: Optional[Union[StrictFloat, StrictInt]] = Field(10, alias="silenceTimeout", description="If no speech is detected in this period, a callback with a 'silence' result is sent.")
    speech_threshold: Optional[Union[StrictFloat, StrictInt]] = Field(10, alias="speechThreshold", description="When speech has ended and a result couldn't be determined based on the audio content itself, this value is used to determine if the speaker is a machine based on the speech duration. If the length of the speech detected is greater than or equal to this threshold, the result will be 'answering-machine'. If the length of speech detected is below this threshold, the result will be 'human'.")
    speech_end_threshold: Optional[Union[StrictFloat, StrictInt]] = Field(5, alias="speechEndThreshold", description="Amount of silence (in seconds) before assuming the callee has finished speaking.")
    machine_speech_end_threshold: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="machineSpeechEndThreshold", description="When an answering machine is detected, the amount of silence (in seconds) before assuming the message has finished playing.  If not provided it will default to the speechEndThreshold value.")
    delay_result: Optional[StrictBool] = Field(False, alias="delayResult", description="If set to 'true' and if an answering machine is detected, the 'answering-machine' callback will be delayed until the machine is done speaking, or an end of message tone is detected, or until the 'detectionTimeout' is exceeded. If false, the 'answering-machine' result is sent immediately.")
    callback_url: Optional[constr(strict=True, max_length=2048)] = Field(None, alias="callbackUrl", description="The URL to send the 'machineDetectionComplete' webhook when the detection is completed. Only for 'async' mode.")
    callback_method: Optional[CallbackMethodEnum] = Field(None, alias="callbackMethod")
    username: Optional[constr(strict=True, max_length=1024)] = Field(None, description="Basic auth username.")
    password: Optional[constr(strict=True, max_length=1024)] = Field(None, description="Basic auth password.")
    fallback_url: Optional[constr(strict=True, max_length=2048)] = Field(None, alias="fallbackUrl", description="A fallback URL which, if provided, will be used to retry the machine detection complete webhook delivery in case `callbackUrl` fails to respond")
    fallback_method: Optional[CallbackMethodEnum] = Field(None, alias="fallbackMethod")
    fallback_username: Optional[constr(strict=True, max_length=1024)] = Field(None, alias="fallbackUsername", description="Basic auth username.")
    fallback_password: Optional[constr(strict=True, max_length=1024)] = Field(None, alias="fallbackPassword", description="Basic auth password.")
    additional_properties: Dict[str, Any] = {}
    __properties = ["mode", "detectionTimeout", "silenceTimeout", "speechThreshold", "speechEndThreshold", "machineSpeechEndThreshold", "delayResult", "callbackUrl", "callbackMethod", "username", "password", "fallbackUrl", "fallbackMethod", "fallbackUsername", "fallbackPassword"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> MachineDetectionConfiguration:
        """Create an instance of MachineDetectionConfiguration from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        # set to None if detection_timeout (nullable) is None
        # and __fields_set__ contains the field
        if self.detection_timeout is None and "detection_timeout" in self.__fields_set__:
            _dict['detectionTimeout'] = None

        # set to None if silence_timeout (nullable) is None
        # and __fields_set__ contains the field
        if self.silence_timeout is None and "silence_timeout" in self.__fields_set__:
            _dict['silenceTimeout'] = None

        # set to None if speech_threshold (nullable) is None
        # and __fields_set__ contains the field
        if self.speech_threshold is None and "speech_threshold" in self.__fields_set__:
            _dict['speechThreshold'] = None

        # set to None if speech_end_threshold (nullable) is None
        # and __fields_set__ contains the field
        if self.speech_end_threshold is None and "speech_end_threshold" in self.__fields_set__:
            _dict['speechEndThreshold'] = None

        # set to None if machine_speech_end_threshold (nullable) is None
        # and __fields_set__ contains the field
        if self.machine_speech_end_threshold is None and "machine_speech_end_threshold" in self.__fields_set__:
            _dict['machineSpeechEndThreshold'] = None

        # set to None if delay_result (nullable) is None
        # and __fields_set__ contains the field
        if self.delay_result is None and "delay_result" in self.__fields_set__:
            _dict['delayResult'] = None

        # set to None if callback_url (nullable) is None
        # and __fields_set__ contains the field
        if self.callback_url is None and "callback_url" in self.__fields_set__:
            _dict['callbackUrl'] = None

        # set to None if callback_method (nullable) is None
        # and __fields_set__ contains the field
        if self.callback_method is None and "callback_method" in self.__fields_set__:
            _dict['callbackMethod'] = None

        # set to None if username (nullable) is None
        # and __fields_set__ contains the field
        if self.username is None and "username" in self.__fields_set__:
            _dict['username'] = None

        # set to None if password (nullable) is None
        # and __fields_set__ contains the field
        if self.password is None and "password" in self.__fields_set__:
            _dict['password'] = None

        # set to None if fallback_url (nullable) is None
        # and __fields_set__ contains the field
        if self.fallback_url is None and "fallback_url" in self.__fields_set__:
            _dict['fallbackUrl'] = None

        # set to None if fallback_method (nullable) is None
        # and __fields_set__ contains the field
        if self.fallback_method is None and "fallback_method" in self.__fields_set__:
            _dict['fallbackMethod'] = None

        # set to None if fallback_username (nullable) is None
        # and __fields_set__ contains the field
        if self.fallback_username is None and "fallback_username" in self.__fields_set__:
            _dict['fallbackUsername'] = None

        # set to None if fallback_password (nullable) is None
        # and __fields_set__ contains the field
        if self.fallback_password is None and "fallback_password" in self.__fields_set__:
            _dict['fallbackPassword'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> MachineDetectionConfiguration:
        """Create an instance of MachineDetectionConfiguration from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return MachineDetectionConfiguration.parse_obj(obj)

        _obj = MachineDetectionConfiguration.parse_obj({
            "mode": obj.get("mode"),
            "detection_timeout": obj.get("detectionTimeout") if obj.get("detectionTimeout") is not None else 15,
            "silence_timeout": obj.get("silenceTimeout") if obj.get("silenceTimeout") is not None else 10,
            "speech_threshold": obj.get("speechThreshold") if obj.get("speechThreshold") is not None else 10,
            "speech_end_threshold": obj.get("speechEndThreshold") if obj.get("speechEndThreshold") is not None else 5,
            "machine_speech_end_threshold": obj.get("machineSpeechEndThreshold"),
            "delay_result": obj.get("delayResult") if obj.get("delayResult") is not None else False,
            "callback_url": obj.get("callbackUrl"),
            "callback_method": obj.get("callbackMethod"),
            "username": obj.get("username"),
            "password": obj.get("password"),
            "fallback_url": obj.get("fallbackUrl"),
            "fallback_method": obj.get("fallbackMethod"),
            "fallback_username": obj.get("fallbackUsername"),
            "fallback_password": obj.get("fallbackPassword")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


